@@ -1,5 +1,5 @@
 ;;; jde-compile.el -- Integrated Development Environment for Java.
-;; $Revision: 1.3 $ $Date: 1999/01/12 23:49:38 $ 
+;; $Revision: 1.1 $ $Date: 2000/08/13 13:43:57 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -260,41 +260,24 @@ source and class files."
   :group 'jde-compile-options
   :type 'boolean)
 
-
+;; Cygwin style paths will not be understood by the java tools.  Call
+;; jde-convert-cygwin-path before passing path information to a
+;; java tool.
 (defun jde-build-path-arg (path-type path-list quote)
 "Build a path argument from a list of paths."
-  (let ((path "")
-	(len (length path-list))
-	(n 0))
-    (while (< n len)
-      (setq path (concat path
-			      (if (> n 0)
-				  jde-classpath-separator)
-			      (elt path-list n)))
-      (setq n (1+ n)))
+  (let ((path 
+	 (mapconcat (lambda (x) x) path-list jde-classpath-separator)))
+    ;; CFH - Call jde-convert-cygwin-path to handle cygwin paths
+    (setq path (jde-convert-cygwin-path path jde-classpath-separator))
     (if quote
-	(setq path (concat "\"" path "\"")))
+        (setq path (concat "\"" path "\"")))
     (setq path (concat path-type " " path))))
 
+
 (defun jde-build-classpath-arg (path-list quote)
 "Build a classpath from a list of paths."
  (jde-build-path-arg "-classpath" path-list quote))
 
-; (defun jde-build-classpath-arg (path-list quote)
-; "Build a classpath from a list of paths."
-;   (let ((classpath "")
-; 	(len (length path-list))
-; 	(n 0))
-;     (while (< n len)
-;       (setq classpath (concat classpath
-; 			      (if (> n 0)
-; 				  jde-classpath-separator)
-; 			      (elt path-list n)))
-;       (setq n (1+ n)))
-;     (if quote
-; 	(setq classpath (concat "\"" classpath "\"")))
-;     (setq classpath (concat "-classpath " classpath))))
-
 (defun jde-get-compile-options ()
 "Constructs a command-line argument string for compiler.
 The string consists of the contents of the jde-compile-options
@@ -449,7 +432,7 @@ history enabled."
     ;; which seems not to be supported--at least on
     ;; the PC.
     (if (and (eq system-type 'windows-nt)
-	     (not jde-xemacsp))
+	     (not jde-xemacsp))	
 	(let ((temp last-nonmenu-event))
 	  ;; The next line makes emacs think that jde-compile
 	  ;; was invoked from the minibuffer, even when it
@@ -464,14 +447,14 @@ history enabled."
 
 ;; Change History
 ;; $Log: jde-compile.el,v $
-;; Revision 1.3  1999/01/12 23:49:38  andyp
-;; Fix classpath separator problems.
+;; Revision 1.1  2000/08/13 13:43:57  michaels
+;; Initial checkin.
 ;;
-;; Revision 1.2  1999/01/12 00:14:04  andyp
-;; kb binary files and fix menu bug.
+;; Revision 1.4  2000/04/10 05:27:30  paulk
+;; Compile command now supports Cygwin-style class paths.
 ;;
-;; Revision 1.1  1999/01/06 06:04:03  andyp
-;; update to 2.1.4
+;; Revision 1.3  1999/01/15 22:04:15  paulk
+;; Added Andy Piper's NT/XEmacs compatibility changes.
 ;;
 ;; Revision 1.2  1998/12/07 01:35:28  paulk
 ;; Updated compile options to reflect changes in command-line options
