@@ -1,5 +1,5 @@
 ;;; jde-db.el -- Debugger mode for jdb.
-;; $Revision: 1.33 $ $Date: 1998/11/22 18:18:36 $ 
+;; $Revision: 1.1 $ $Date: 2000/08/13 13:43:58 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -35,28 +35,61 @@
 ;;; Code:
 
 (require 'gud)
+(require 'jde-parse)
 
 ;; ======================================================================
 ;; jde-db variables
 
-(defcustom jde-db-debugger (cons "jdb" "Executable")
-"*Specify debugger.
-Enter the path name of debugger, if debugger ia an executable; otherwise,
-the fully qualified package name of the debugger class."
+(makunbound 'jde-db-debugger)
+(defcustom jde-db-debugger (cons "Other" (cons "jdb" "Executable"))
+"Specify the debugger you want to use to debug Java programs.
+Select JDEbug, if you want to use the JDE's builtin debugger. Otherwise,
+select Other Other may be an executable, like jdb, or a Java class. If Other 
+is a class, the JDE launches a vm to run the debugger. If you choose Other,
+ select the debugger's type (executable or class) and enter the path name 
+of the debugger, if Other is an executable, or a  fully qualified package name,
+if Other is a class."
   :group 'jde-project
   :type '(cons
-	  (string :tag "Name")
 	  (radio-button-choice :format "%t \n%v"
-			       :tag "Debugger type is "
+			       :tag "Debugger "
+			       (const "JDEbug")
+			       (const "Other"))
+	  (cons :tag "Other Debugger Info"
+	   (string :tag "Name")
+	   (radio-button-choice :format "%t \n%v"
+			       :tag "Type "
 		 (const "Executable")
 		 (const "Class"))))
+  :set '(lambda (sym val)
+	  (mapc 
+	   (lambda (buff)
+	     (save-excursion	       
+	       (set-buffer buff)
+	       (if (string= (car val) "JDEbug")
+		   (jde-bug-install-jdebug-menu)
+		 (jde-bug-remove-jdebug-menu))))
+	     (jde-get-java-source-buffers))
+	  (set-default sym val)))
 
-(defcustom jde-db-source-directories (list "d:/jdk1.2/src/")
+(defcustom jde-db-source-directories nil
   "*List of source directory paths.
-jde-db uses this list to locate source files visited by the debugger
-while stepping through an application."
+The JDE uses this list to locate source files corresponding
+to class files when debugging or building applications.
+When entering paths in the custom buffer, enter each path as a separate
+item in a separate edit field. Do NOT put more than one path in the
+same edit field. You'll only confuse JDE."
   :group 'jde-project
-  :type '(repeat (string :tag "Path")))
+  :type '(repeat (string :tag "Path"))
+  :set '(lambda (sym val)
+	  (set-default 
+	   sym 
+	   (mapcar
+	    (lambda (path)
+	      (if (not (string= (substring path (- (length path) 1)) "/"))
+	       (concat path "/")
+	       path))
+	    val))))
 
 
 (defcustom jde-db-mode-hook nil
@@ -93,9 +126,6 @@ the `jde-db-option' variable group."
   :group 'jde-project
   :type 'boolean)
 
-(defvar jde-db-interactive-vm-args ""
-"Vm arguments read from the minibuffer.")
-
 (defvar jde-db-interactive-vm-arg-history nil
 "History of vm arguments read from the minibuffer")
 
@@ -104,9 +134,6 @@ the `jde-db-option' variable group."
   :group 'jde-project
   :type 'boolean)
 
-(defvar jde-db-interactive-app-args ""
-"Application arguments read from the minibuffer.")
-
 (defvar jde-db-interactive-app-arg-history nil
 "History of application arguments read from the minibuffer")
 
@@ -247,12 +274,12 @@ the command line."
 
 
 ;;;###autoload
-(defun jde-db-set-debugger (debugger is-executable)
+(defun jde-db-set-debugger (name is-executable)
   "Specify the pathname of the debugger, if an executable, or the
 debugger's fully qualified class name, if a class."
   (interactive
    "sEnter name of Java interpreter: \nsIs %s executable? (yes): ")
-  (let ((db debugger)
+  (let ((db name)
 	(type
 	 (if (stringp is-executable)
 	     (if (or
@@ -261,7 +288,7 @@ debugger's fully qualified class name, if a class."
 		 "Executable"
 	       "Class")
 	   "Executable")))
-    (setq jde-db-debugger (cons db type))))
+    (setq jde-db-debugger (cons "Other" (cons db type)))))
 
 ;;;###autoload
 (defun jde-db-set-args (args)
@@ -287,19 +314,28 @@ Run Option panel variables."
 	       (cons "kilobytes" "k")
 	       (cons "megabytes" "m"))))
 
+
+    ;; Set classic-mode option
+    (if jde-run-classic-mode-vm
+	(setq options
+	       (list "-classic")))
+
     ;; Set the classpath option. Use the local
     ;; classpath, if set; otherwise, the global
     ;; classpath.
     (if jde-db-option-classpath
 	(setq options
-	      (list "-classpath"
-		    (jde-run-build-classpath-arg
-		     jde-db-option-classpath)))
+	      (nconc options 
+		     (list "-classpath"
+			   (jde-run-build-classpath-arg
+			    jde-db-option-classpath))))
       (if jde-global-classpath
 	  (setq options
-		(list "-classpath"
-		      (jde-run-build-classpath-arg
-		       jde-global-classpath)))))
+		(nconc options
+		       (list "-classpath"
+			     (jde-run-build-classpath-arg
+			      jde-global-classpath))))))
+
 
     ;; Set the verbose options.
     (let ((print-classes-loaded
@@ -323,7 +359,7 @@ Run Option panel variables."
 	    (let ((prop (nth n jde-db-option-properties)))
 	      (setq options 
 		    (nconc options
-			   (list (concat " -D" (car prop) "=" (cdr prop))))))    
+			   (list (concat "-D" (car prop) "=" (cdr prop))))))    
 	    (setq n (1+ n)))))
 
     ;; Set heap size options.
@@ -463,11 +499,15 @@ line number information is available.")
 
 ;; Thanks to "David J. Biesack" <sasdjb@unx.sas.com> for this function
 ;; and its use in jde-db-marker-filter.
+;; Amended by "Patrick J. McNerthney" <pat@mcnerthney.com> to allow
+;; package names to begin underscores.
 (defun jde-db-make-qualified-class-name-regexp (class)
 "Constructs a regular expression to extract a qualified class name from a jdb
 breakpoint message."
-  (concat "\\b\\(\\(\\(\\w*\\.\\)*\\)" class "\\)\\b"))
+(concat "\\(\\(\\(\\(\\w\\|[_]\\)*\\.\\)*\\)" class "\\)\\(\\b\\|\\$\\)"))
 
+;; David Biesack's original expression.
+;;(concat "\\b\\(\\(\\(\\(\\w\\|[_]\\)*\\.\\)*\\)" class "\\)\\(\\b\\|\\$\\)"))
 
 ;; There's no guarantee that Emacs will hand the filter the entire
 ;; marker at once; it could be broken up across several strings.  We
@@ -511,11 +551,14 @@ breakpoint message."
 
 	  ;; Extract package path from input.
 	  (setq jde-db-last-package "")
-	  (and (string-match (jde-db-make-qualified-class-name-regexp class) marker)
-	       (setq jde-db-last-package
-		     (substring marker (match-beginning 2) (match-end 2))))
+	  (let ((case-fold-search nil)) ;; Make sure search is case-sensitive
+	    (and (string-match (jde-db-make-qualified-class-name-regexp class) marker)
+		 (setq jde-db-last-package
+		       (substring marker (match-beginning 2) (match-end 2))))
 
-	  ;; (message "jde-db package: %s" jde-db-last-package)
+	    ;; (message "jde-db package: %s. marker = %s" jde-db-last-package marker)
+	    ;;(message "case-fold-search = %s" (if case-fold-search "true" "false"))
+	    )
 
 	  (setq output (concat premarker "Stopped at line " (int-to-string line-no) " in "
 			       class ".java"))
@@ -554,29 +597,11 @@ breakpoint message."
 
     output))
 
+
 (defun jde-db-contains-file-p (dir file)
   "Return t if DIR contains FILE."
-  (let ((files (directory-files dir)))
-    (catch 'found
-      (let ((n (- (length files) 1)))
-	(while (>= n 0)
-	  (if (string= file (elt files n))
-	      (throw 'found t))
-	  (setq n (- n 1)))))))
+  (file-exists-p (expand-file-name file dir)))
 
-(defun jde-db-contains-package-p (dir package)
-  "Return t if DIR contains PACKAGE."
-  (let ((files (directory-files dir))
-	(pkg-root (substring package
-			     0
-			     (string-match "\\." package))))
-    (catch 'found
-      (let ((n (- (length files) 1)))
-	(while (>= n 0)
-	  (let ((curr-file (elt files n)))
-	    (if (string= pkg-root curr-file)
-		(throw 'found t)))
-	  (setq n (- n 1)))))))
 
 (defun jde-db-pkg-to-path (package)
   "Return PACKAGE as a directory path."
@@ -588,6 +613,17 @@ breakpoint message."
     output))
       
 
+(defun jde-db-contains-package-p (dir package)
+   "Return t if DIR contains PACKAGE."
+   (let ((pkg-path (jde-db-pkg-to-path package)))
+     (file-exists-p (expand-file-name pkg-path dir))))
+
+; (defun jde-test (dir package)
+;   (interactive
+;    "sDir: \nsPackage: ")
+;   (message (if (jde-db-contains-package-p dir package) "true" "false")))
+
+
 (defun jde-db-search-src-dirs (file package)
   "Return the directory containing the source FILE for a class in PACKAGE."
   (catch 'found
@@ -625,24 +661,55 @@ and create such a submap in that new map."
 
 
 (defun jde-db-find-file (f)
-  (save-excursion
-    (let ((buf (find-file-noselect 
-		(concat 
-		 (jde-db-search-src-dirs f jde-db-last-package) f))))
-      (set-buffer buf)
-      (if (string-match "XEmacs\\|Lucid" emacs-version)
-	  (if (and (boundp 'current-menubar)
-		   current-menubar)
-	      (if (fboundp 'add-submenu)
-		  (add-submenu nil (jde-db-xemacs-menu))
-		(add-menu nil "Jdb" jde-db-xemacs-menu)))
-	(progn
-	  (jde-db-make-debug-menu)
-	  (local-set-key [menu-bar debug refresh] nil)
-	  (local-set-key [menu-bar debug stepi] nil)
-	  (local-set-key [menu-bar debug up] '("Up Stack" . gud-up))
-	  (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))))
-      buf)))
+  ;;(interactive "sFile: " )
+  (let ((source-dir 
+	 (jde-db-search-src-dirs f jde-db-last-package)))
+    (if source-dir
+	(let ((source-file
+	       (concat source-dir f)))	
+	  (save-excursion
+	    (let ((buf (find-file-noselect source-file)))
+	      (set-buffer buf)
+	      (if (string-match "XEmacs\\|Lucid" emacs-version)
+		  (if (and (boundp 'current-menubar)
+			   current-menubar)
+		      (if (fboundp 'add-submenu)
+			  (add-submenu nil (jde-db-xemacs-menu))
+			(add-menu nil "Jdb" jde-db-xemacs-menu)))
+		(progn
+		  (jde-db-make-debug-menu)
+		  (local-set-key [menu-bar debug refresh] nil)
+		  (local-set-key [menu-bar debug stepi] nil)
+		  (local-set-key [menu-bar debug up] '("Up Stack" . gud-up))
+		  (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))))
+	      buf)))
+	  (message (concat "Error: could not find %s. "
+			   "See jde-db-source-directories.") f)
+	  (if jde-xemacsp (window-buffer) nil))))
+
+(defun jde-find-class-source (class)
+  "*Find the source file for a specified class.
+CLASS is the fully qualified name of the class. This
+function searchs the source file paths specified by 
+`jde-db-source-directories' for the source file 
+corresponding to CLASS. If it finds the source file,
+it opens the file in a buffer."
+  (interactive "sClass: ")
+  (string-match "^\\(\\(\\(\\w\\|[_]\\)*[.]\\)*\\)\\(\\(\\w\\|[_]\\)+$\\)" class)  
+  (let* ((package-name
+	  (substring class (match-beginning 1) (match-end 1)))
+	 (class-name
+	  (substring class (match-beginning 4) (match-end 4)))
+	 (file-name (concat class-name ".java"))
+	 (source-dir
+	  (jde-db-search-src-dirs
+	   file-name
+	   package-name)))
+    (if source-dir
+	(find-file (concat source-dir file-name))
+      (message (concat "JDE error: Could not find source for %s. "
+		       "See `jde-db-source-directories' for more information." )
+		       class))))
 
 (defvar jde-db-minibuffer-local-map nil
   "Keymap for minibuffer prompting of jdb startup command.")
@@ -679,9 +746,15 @@ and create such a submap in that new map."
 						(car frame)))))
 	 ((eq key ?c)
 	  (setq subst (concat (jde-db-get-package)
-			      (class-from-file-name (if insource
-							(buffer-file-name)
-						      (car frame))))))
+			      (or
+			       (if insource
+				   (jde-db-get-class)
+				 (save-excursion
+				   (set-buffer (gud-find-file (car frame)))
+				   (jde-db-get-class)))
+			       (error "Could not determine containing class.")))))
+
+
 
 	 ((eq key ?d)
 	  (setq subst (file-name-directory (if insource
@@ -695,12 +768,15 @@ and create such a submap in that new map."
 					      (1+ (count-lines 1 (point)))))
 			(cdr frame))))
 	 ((eq key ?e)
-	  (setq subst (find-c-expr)))
+	  (setq subst (if (fboundp 'gud-find-c-expr)
+			  (gud-find-c-expr)
+			(find-c-expr))))
 	 ((eq key ?a)
 	  (setq subst (gud-read-address)))
 	 ((eq key ?p)
 	  (setq subst (if arg (int-to-string arg) ""))))
-	(setq result (concat result group1 (if (integerp subst) (int-to-string subst) subst)))))
+	(setq result (concat result group1 
+			     (if (integerp subst) (int-to-string subst) subst)))))
     ;; There might be text left in STR when the loop ends.
     (concat result str)))
 
@@ -749,20 +825,20 @@ we're in the jdb buffer)."
       (jde-run-parse-args
        (read-from-minibuffer
 	"Vm args: "
-	jde-db-interactive-vm-args
+	(car jde-db-interactive-vm-arg-history)
 	nil nil
-	'(jde-db-interactive-vm-arg-history . 1)))))
+	'jde-db-interactive-vm-arg-history))))
 
 (defun jde-db-get-app-args-from-user ()
   (if jde-db-read-app-args
       (jde-run-parse-args
        (read-from-minibuffer
 	"Application args: "
-	jde-db-interactive-app-args
+	(car jde-db-interactive-app-arg-history)
 	nil nil
-	'(jde-db-interactive-app-arg-history . 1)))))
+	'jde-db-interactive-app-arg-history))))
 
-(defun jde-db-init(app-class marker-filter find-file)
+(defun jde-db-init (app-class marker-filter find-file)
   (let ((debug-buf-name (concat "*debug" app-class "*"))
 	(source-directory default-directory)
 	(working-directory (if (string= jde-run-working-directory "")
@@ -770,10 +846,10 @@ we're in the jdb buffer)."
 			     jde-run-working-directory)))
     (if (not (comint-check-proc debug-buf-name))
 	(let* ((debug-buffer (get-buffer-create debug-buf-name))
-	       (program (if (string= (cdr jde-db-debugger) "Executable")
-			   (car jde-db-debugger)
+	       (program (if (string= (cdr (cdr jde-db-debugger)) "Executable")
+			   (car (cdr jde-db-debugger))
 			 jde-run-java-vm))
-	       (prog-args (if (string= (cdr jde-db-debugger) "Executable")
+	       (prog-args (if (string= (cdr (cdr jde-db-debugger)) "Executable")
 			      (append 
 			       (jde-db-get-vm-args)
 			       (jde-db-get-vm-args-from-user)
@@ -781,7 +857,7 @@ we're in the jdb buffer)."
 			       jde-db-option-application-args
 			       (jde-db-get-app-args-from-user))
 			    (append
-			     (list (car jde-db-debugger))
+			     (list (car (cdr jde-db-debugger)))
 			     (jde-db-get-vm-args)
 			     (jde-db-get-vm-args-from-user)
 			     (list app-class)
@@ -792,7 +868,9 @@ we're in the jdb buffer)."
 				(jde-run-make-arg-string prog-args) "\n\n")))
 	  (save-excursion
 	    (set-buffer debug-buffer)
-	    (erase-buffer)
+	    ;; Do not erase the last transcript; user may wish to view it.
+	    ;; (erase-buffer)
+	    (goto-char (point-max))
 	    (cd working-directory)
 	    (insert (concat "cd " working-directory "\n"))
 	    (insert command-string)
@@ -823,14 +901,31 @@ we're in the jdb buffer)."
 buffer."
   (save-excursion
     (goto-char (point-min))
-    (if (re-search-forward "\\(package\\) *\\([^ ]*\\) *;" (point-max) t)
+    (if (re-search-forward "^[ \t]*\\<\\(package\\) *\\([^ \t\n]*\\) *;" (point-max) t)
 	(concat (buffer-substring-no-properties (match-beginning 2) (match-end 2))
 		"."))))
 
+(defun jde-db-get-class () "Lookups and return fully qualified class
+name, e.g. A$B if point is in inner class B of A."
+  (interactive)
+  (let ((class-info (jde-parse-get-innermost-class-at-point)))
+    (if class-info
+      (save-excursion
+	(goto-char (cdr class-info))
+	(let ((parent (jde-db-get-class)))
+	(if (not parent)
+	    (car class-info)
+	    (concat parent "$" (car class-info))))))))
+
+
 ;;;###autoload		   
 (defun jde-db ()
-  "Run jdb on Java class whose source resides in the current buffer.
-This command creates a command buffer named *gud-CLASS* for the debug session.
+  "Run jdb on Java application whose source resides in the current buffer.
+This command determines the main class of the application either from
+the variable `jde-run-application-class' or from the source in the current 
+buffer. If `jde-run-application-class' does not specify a class, the main class
+is assumed to be the class defined by the current source buffer. This command
+creates a command buffer named *gud-CLASS* for the debug session.
 CLASS is the name of the class you are debugging."
   (interactive)
 
@@ -880,12 +975,13 @@ CLASS is the name of the class you are debugging."
 
 ;;;###autoload
 (defun jde-db-set-source-paths (paths)
- "Set the source path list used by JDE to locate Java source files visited
-by the debugger. PATHS is a list of source paths separated by colons or semicolons,
-depending on the operating system.. Note that for packages, you need enter only the 
-directory containing the top-level package. For example, including '../jdk1.1/src/'
- enables the JDE to  locate all source files provided with JDK1.1. Note also that the 
-paths must end in a slash."
+ "Set the source path list used by JDE to locate Java source files
+visited by the debugger. PATHS is a list of source paths separated by
+colons or semicolons, depending on the operating system.. Note that
+for packages, you need enter only the directory containing the
+top-level package. For example, including '../jdk1.1/src/' enables the
+JDE to locate all source files provided with JDK1.1. Note also that
+the paths must end in a slash."
  (interactive
   "sEnter source paths: ")
  (let ((m 0)
@@ -907,11 +1003,240 @@ paths must end in a slash."
    (setq jde-db-source-directories (nreverse jde-db-source-directories))))
 
 
+(defun jde-db-applet-init(applet-class marker-filter find-file applet-doc)
+  (let ((debug-buf-name (concat "*debug" applet-class "*"))
+	(source-directory default-directory)
+	(working-directory (if (string= jde-run-working-directory "")
+			       default-directory
+			     jde-run-working-directory)))
+    (if (not (comint-check-proc debug-buf-name))
+	(let* ((debug-buffer (get-buffer-create debug-buf-name))
+	       (program "appletviewer")
+	       (prog-args (append
+			   (jde-get-appletviewer-options)
+			   (list "-debug" applet-doc)))
+	       (command-string (concat
+				program " "
+				(jde-run-make-arg-string prog-args) "\n\n")))
+	  (save-excursion
+	    (set-buffer debug-buffer)
+	    (erase-buffer)
+	    (cd working-directory)
+	    (insert (concat "cd " working-directory "\n"))
+	    (insert command-string)
+	    (comint-mode))
+	  (comint-exec debug-buffer applet-class program nil prog-args)
+	  (pop-to-buffer debug-buffer)
+	  (cd source-directory)
+	  (gud-mode)
+	  (make-local-variable 'gud-marker-filter)
+	  (setq gud-marker-filter marker-filter)
+	  (make-local-variable 'gud-find-file)
+	  (setq gud-find-file find-file)
+	  (set-process-filter (get-buffer-process (current-buffer)) 'gud-filter)
+	  (set-process-sentinel (get-buffer-process (current-buffer)) 'gud-sentinel)
+	  (gud-set-buffer)
+	  (if jde-db-startup-commands
+	    (mapc 'gud-basic-call jde-db-startup-commands)
+	   (when jde-db-set-initial-breakpoint
+	    (gud-basic-call (concat "stop in " applet-class ".init"))
+	    (gud-basic-call "run")))
+	  )
+      (message "An instance of %s is running." applet-class)			
+      (pop-to-buffer debug-buf-name))))			   
+	  
+   
+(defun jde-db-applet-internal (applet-doc)
+  ;; test for XEmacs
+  (if (string-match "XEmacs\\|Lucid" emacs-version)
+      (gud-overload-functions
+       '((gud-marker-filter . jde-db-marker-filter)
+	 (gud-find-file . jde-db-find-file)
+	 )))
+
+  (let ((applet-class jde-run-application-class))
+    (if (or
+	 (not applet-class)
+	 (string= applet-class ""))
+	(setq applet-class
+	      (concat (jde-db-get-package)
+		      (file-name-sans-extension 
+		       (file-name-nondirectory (buffer-file-name))))))
+    (jde-db-applet-init applet-class 'jde-db-marker-filter 'jde-db-find-file applet-doc))
+
+  (jde-db-def gud-break  "stop at %c:%l"  "\C-b" "Set breakpoint at current line.")
+  (jde-db-def gud-remove "clear %c:%l"    "\C-d" "Remove breakpoint at current line")
+  (jde-db-def gud-step   "step"           "\C-s" "Step one source line with display.")
+  (jde-db-def gud-next   "next"           "\C-n" "Step one line (skip functions).")
+  (jde-db-def gud-cont   "cont"           "\C-r" "Continue with display.")
+  (jde-db-def gud-up     "up %p"          "<" "Up N stack frames (numeric arg).")
+  (jde-db-def gud-down   "down %p"        ">" "Down N stack frames (numeric arg).")
+  (jde-db-def gud-print  "print %e"       "\C-p" "Print object.")
+  (local-set-key [menu-bar debug up]   '("Up Stack" . gud-up))
+  (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))
+  (make-local-variable 'comint-prompt-regexp)
+  (setq comint-prompt-regexp "\\(^> *\\)\\|\\(^.*\\[[0-9]*\\] *\\)")
+  (make-local-variable 'paragraph-start)
+  (setq paragraph-start comint-prompt-regexp)
+  (run-hooks 'jde-db-mode-hook)
+  )
+
+;;;###autoload 
+(defun jde-db-applet (&optional doc) 
+  "Runs an applet in the debugger. This function prompts you to enter
+the path of an html document that displays the applet. If you 
+do not enter a path, this function next checks
+whether `jde-run-applet-doc' specifies a document. If so, it displays
+that specified document. Next, it checks whether the current directory
+contains any html files. If so, it displays the first html file that
+it finds. If if cannot find an html file, it signals an error.  This
+function runs appletviewer in jdb to permit debugging. On startup, it
+sets a breakpoint in the init method of the class specified by 
+`jde-run-application-class' or in the class corresponding to the Java
+file in the current buffer."
+  (interactive
+   (let ((insert-default-directory nil))
+     (list (read-file-name "Applet doc: " nil nil nil jde-run-applet-document))))
+  (setq jde-run-applet-document doc)
+  (let ((applet-doc (if (and jde-run-applet-document
+			     (not (string= jde-run-applet-document "")))
+			jde-run-applet-document
+		      (if (and jde-run-applet-doc
+			       (not (string= jde-run-applet-doc "")))
+			    jde-run-applet-doc
+			  (car (jde-run-find-html-files))))))
+    (if applet-doc 
+	(jde-db-applet-internal applet-doc) 
+      (signal 'error "Could not find html document to display applet."))))
+
+
+(defun jde-db-menu-debug-applet ()
+  (interactive)
+  (jde-db-applet))
+
 (provide 'jde-db)
 
 
 ;; Change History
-;; $Log: jde-db.el $
+;; $Log: jde-db.el,v $
+;; Revision 1.1  2000/08/13 13:43:58  michaels
+;; Initial checkin.
+;;
+;; Revision 1.63  2000/06/12 08:37:43  paulk
+;; Now displays JDEbug menu when running XEmacs.
+;;
+;; Revision 1.62  2000/05/10 05:41:32  paulk
+;; The JDEbug menu now appears or disappears when you select or deselect JDEbug as the current debugger.
+;;
+;; Revision 1.61  2000/03/16 05:08:25  paulk
+;; Added JDEbug option to jde-db-debugger.
+;;
+;; Revision 1.60  2000/03/08 05:40:01  paulk
+;; jde-db-format-command now signals an error it it cannot determine the containing class.
+;;
+;; Revision 1.59  2000/02/10 02:50:32  paulk
+;; Replaced jde expand file name function with expand-file-name.
+;;
+;; Revision 1.58  2000/02/01 04:08:12  paulk
+;; Modified the Jdb->Set Breakpoint command (gud-break) to set breakpoints correctly
+;; in inner classes.
+;;
+;; Revision 1.57  2000/01/15 08:01:52  paulk
+;; Reimplemented directory search functions.
+;;
+;; Revision 1.56  1999/11/16 05:58:17  paulk
+;; Added trace method commands and skeletons for trace class and cancel
+;; trace commands.
+;;
+;; Revision 1.55  1999/11/04 05:49:10  paulk
+;; Amended jde-db-make-qualified-class-name-regexp to permit package names to begin
+;; with non-word characters, e.g., underscores. Contributed by "Patrick J. McNerthney"
+;; <pat@mcnerthney.com>.
+;;
+;; Revision 1.54  1999/09/28 04:06:59  paulk
+;; Supplied missing left parentheses.
+;;
+;; Revision 1.53  1999/09/05 04:33:28  paulk
+;; Added support for running vm in classic mode.
+;;
+;; Revision 1.52  1999/03/10 16:55:02  paulk
+;; Fixed jde-db-find-file to return the current buffer if it cannot find a file and
+;; XEmacs is the editor.
+;;
+;; Revision 1.51  1999/03/06 00:55:38  paulk
+;; Changed default value of jde-db-source-directories to be nil.
+;;
+;; Revision 1.50  1999/03/06 00:44:08  paulk
+;; Make sure that case-sensitive matching is used when extracting package names from
+;; debugger breakpoint messages.
+;;
+;; Revision 1.49  1999/02/26 15:52:52  paulk
+;; Catch non-existant directory errors when searching for source
+;; files and packages. Thanks to Thanh Nguyen <Thanh.Nguyen@Eng.Sun.COM>
+;; for finding and providing a fix for this bug.
+;;
+;; Revision 1.48  1999/02/25 15:24:43  paulk
+;; Fixed jde-db-find-file so that it displays an error when it cannot find a file instead of
+;; opening an empty source buffer.
+;;
+;; Provided a set-value function for jde-db-source-directories that appends a slash to
+;; the end of each path if the path does not already end in a slash.
+;;
+;; Defined a new command, jde-find-class-source, that finds and opens the source file
+;; for a specified class.
+;;
+;; Improved the regular expression used by jde-db-get-package to ignore tabs at the
+;; beginning of a line.
+;;
+;; Revision 1.47  1999/02/15 02:02:35  paulk
+;; Forgot to concatenate in last fix.
+;;
+;; Revision 1.46  1999/02/15 00:52:44  paulk
+;; Fixed bug in qualified-class-name-regexp.
+;;
+;; Revision 1.45  1999/02/10 18:35:51  paulk
+;; Added support for appletviewer -encoding and -J options.
+;;
+;; Revision 1.44  1999/02/08 17:18:17  paulk
+;; jde-db-applet now supports file completion and remembers the last path entered.
+;;
+;; Revision 1.43  1999/02/06 03:55:11  paulk
+;; Fixed bug and generalized regular expression in jde-db-make-qualified-class-name-regexp.
+;;
+;; Revision 1.42  1999/02/03 18:12:03  paulk
+;; Fixed regular expression in jde-db-get-package to eliminate spurious hits, e.g.
+;; commented out package statements. Thanks to Frederic Baumann <baumann@ilog.fr>
+;; for reporting this bug.
+;;
+;; Revision 1.41  1999/02/03 17:48:34  paulk
+;; Patched jde-db-get-app-args-from-user to remember arguments.
+;; Thanks to Brian Burton <brian@burton-computer.com>
+;;
+;; Revision 1.40  1999/02/03 17:41:56  paulk
+;; Fixed jde-db-make-qualified-class-name-regexp to handle packages with underscores.
+;; Thanks to Brian Burton <brian@burton-computer.com>.
+;;
+;; Revision 1.39  1999/02/03 17:26:46  paulk
+;; Changed jde-db-make-qualified-class-name-regexp to handle inner classes.
+;; Thanks to Michael Lepore <lepore@process.com> for this fix.
+;;
+;; Revision 1.38  1999/02/03 01:53:49  paulk
+;; Fixed jde-db-applet to check the current directory for the html file to run.
+;;
+;; Revision 1.37  1999/02/02 16:06:01  paulk
+;; Added the jde-db-applet command. This command allows you to debug an applet, using
+;; appletviewer.
+;;
+;; Revision 1.36  1999/02/02 15:25:28  paulk
+;; Removed unwanted space in -D (properties) debug option.
+;;
+;; Revision 1.35  1999/01/17 00:36:43  paulk
+;; Now uses gud-find-c-expr or find-c-expr, whichever is bound.
+;;
+;; Revision 1.34  1999/01/13 22:18:08  paulk
+;; Added Andy Piper's NT/XEmacs 21 compatibility changes.
+;; Changed find-c-expr to gud-findc-expr
+;;
 ;; Revision 1.33  1998/11/22 18:18:36  paulk
 ;; Made comint-prompt-regexp and  paragraph-start local variables.
 ;;
@@ -1049,4 +1374,4 @@ paths must end in a slash."
 ;; Initial revision
 ;;
 
-;;; end of jde-db.el
+;;; end of jde-db.el
\ No newline at end of file
