@@ -1,5 +1,5 @@
 ;; jde-run.el --- runs the Java app in the current buffer.
-;; $Revision: 1.17 $ $Date: 1998/06/30 21:10:28 $
+;; $Revision: 1.1 $ $Date: 2000/08/13 13:44:01 $
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -50,6 +50,13 @@ method."
   :group 'jde-project
   :type 'string)
 
+(defcustom jde-run-working-directory ""
+  "*Path of the working directory for this application.
+If you specify a path, the JDE launches the application from the
+directory specified by the path."
+  :group 'jde-project
+  :type 'string)
+
 
 (defcustom jde-run-java-vm "java"
   "*Specify Java interpreter for non-Windows platforms."
@@ -66,6 +73,13 @@ confusing? It is, but it works."
   :group 'jde-project
   :type 'string)
 
+
+(defcustom jde-run-classic-mode-vm nil
+"Runs applications in the classic (i.e., not HotSpot) mode."
+  :group 'jde-project
+  :type 'boolean)
+
+
 (defcustom jde-run-read-vm-args nil
 "*Read vm arguments from the minibuffer.
 If this variable is non-nil, the jde-run command reads vm arguments
@@ -74,9 +88,6 @@ the `jde-run-option' variable group."
   :group 'jde-project
   :type 'boolean)
 
-(defvar jde-run-interactive-vm-args ""
-"Vm arguments read from the minibuffer.")
-
 (defvar jde-run-interactive-vm-arg-history nil
 "History of vm arguments read from the minibuffer")
 
@@ -85,9 +96,6 @@ the `jde-run-option' variable group."
   :group 'jde-project
   :type 'boolean)
 
-(defvar jde-run-interactive-app-args ""
-"Application arguments read from the minibuffer.")
-
 (defvar jde-run-interactive-app-arg-history nil
 "History of application arguments read from the minibuffer")
 
@@ -225,30 +233,61 @@ the command line."
   :group 'jde-run-options
   :type '(repeat (string :tag "Argument")))
 
-(defcustom jde-run-applet-viewer "appletviewer"
+(defcustom jde-run-applet-viewer ""
   "*Specify name of viewer to use to display page containing the applet."
   :group 'jde-project
   :type 'string)
 
-(defcustom jde-run-applet-doc "index.html"
+(defcustom jde-run-applet-doc ""
   "*Specify name of document containing applet to be viewed.
 If no document is specified, JDE assumes that the document name is
-APPLET.html, where APPLET is the name of the applet to be viewed. The
-default value is index.html."
+APPLET.html, where APPLET is the name of the applet to be viewed."
   :group 'jde-project
   :type 'string)
 
 
+(defcustom jde-appletviewer-option-encoding ""
+"*Specify encoding of the HTML file displayed by the appletviewer."
+  :group 'jde-run-options
+  :type 'string)
+
+
+(defcustom jde-appletviewer-option-vm-args nil
+  "*Specify arguments (e.g., -Xmx16m) to the vm that runs appletviewer.
+This option allows you to set the environment of the
+virtual machine that runs appletviewer."
+  :group 'jde-run-options
+  :type '(repeat (string :tag "Argument")))
+
+
+(defcustom jde-run-executable ""
+  "*Specifies the executable to be run by the JDE's run command.
+If you do not specify an executable, the JDE runs the vm specified
+by `jde-run-java-vm' or `jde-run-java-vm-w'."
+  :group 'jde-project
+  :type 'string
+)
+
+(defcustom jde-run-executable-args nil
+  "*Specify arguments to be passed to the application executable.
+This option allows you to specify one or more arguments to be passed
+to the executable specified by `jde-run-executable'."
+  :group 'jde-run-options
+  :type '(repeat (string :tag "Argument")))
+
+
 (defun jde-run-parse-args (s)
-"Converts a string of command-line arguments to a list of arguments."
-  (let ((n (string-match "[a-zA-Z0-9\\.:=;%+\\_/-]+" s))
-	(i 0)
-	(tokens '()))
-    (while n
-      (setq tokens (append tokens (list (match-string 0 s))))
-      (setq n (match-end 0))
-      (setq n (string-match "[a-zA-Z0-9\\.:=;%+\\_/-]+" s n)))
-    tokens))
+ "Converts a string of command-line arguments to a list of arguments.
+Any substring that is enclosed in single or double quotes or does not include
+whitespace is considered a parameter."
+   (let ((n (string-match "[^\" ][^ ]*\\|\"[^\"]*\"\\|'[^']*'" s))
+ 	(i 0)
+ 	(tokens '()))
+     (while n
+       (setq tokens (append tokens (list (match-string 0 s))))
+       (setq n (match-end 0))
+       (setq n (string-match "[^\" ][^ ]*\\|\"[^\"]*\"\\|'[^']*'" s n)))
+     tokens))
 
 (defun jde-run-make-arg-string (args)
 "Converts a list of command-line arguments to a string of arguments."
@@ -332,10 +371,11 @@ panel to specifying the applet document."
     (while (< n len)
       (setq classpath (concat classpath
 			      (if (> n 0)
-				  path-separator)
+				  jde-classpath-separator)
 			      (elt path-list n)))
       (setq n (1+ n)))
-    classpath))
+      (jde-convert-cygwin-path classpath jde-classpath-separator)))
+
 
 (defun jde-run-get-vm-args ()
   "Builds a command-line argument string to pass to the Java vm.
@@ -347,19 +387,27 @@ Run Option panel variables."
 	       (cons "kilobytes" "k")
 	       (cons "megabytes" "m"))))
 
+    ;; Set classic-mode option
+    (if jde-run-classic-mode-vm
+	(setq options
+	      (nconc
+	       (list "-classic"))))
+
     ;; Set the classpath option. Use the local
     ;; classpath, if set; otherwise, the global
     ;; classpath.
     (if jde-run-option-classpath
 	(setq options
-	      (list "-classpath"
-		    (jde-run-build-classpath-arg
-		     jde-run-option-classpath)))
+	      (nconc options 
+		     (list "-classpath"
+			   (jde-run-build-classpath-arg
+			    jde-run-option-classpath))))
       (if jde-global-classpath
 	  (setq options
-		(list "-classpath"
-		      (jde-run-build-classpath-arg
-		       jde-global-classpath)))))
+		(nconc options
+		       (list "-classpath"
+			     (jde-run-build-classpath-arg
+			      jde-global-classpath))))))
 
     ;; Set the verbose options.
     (let ((print-classes-loaded
@@ -367,7 +415,7 @@ Run Option panel variables."
 	  (print-memory-freed
 	   (nth 1 jde-run-option-verbose))
 	  (print-jni-info
-	   (nth 2 jde-run-option-verbose)))
+   (nth 2 jde-run-option-verbose)))
       (if print-classes-loaded
 	  (setq options (nconc options (list "-v"))))
       (if print-memory-freed
@@ -383,7 +431,7 @@ Run Option panel variables."
 	    (let ((prop (nth n jde-run-option-properties)))
 	      (setq options 
 		    (nconc options
-			   (list (concat " -D" (car prop) "=" (cdr prop))))))    
+			   (list (concat "-D" (car prop) "=" (cdr prop))))))    
 	    (setq n (1+ n)))))
 
     ;; Set heap size options.
@@ -483,41 +531,74 @@ Run Option panel variables."
 	      
     options))
 
+
 ;;;###autoload
-(defun jde-run()
-  "Runs the Java program named by jde-run-application-class in
-a buffer, piping output from the program to the buffer and 
-input from the buffer to the program."
+(defun jde-run ()
+  "Run the Java application specified by `jde-run-executable', if
+not the null string. Otherwise run the class specified by 
+`jde-run-application-class', if non-null; otherwise the class in
+the current buffer. This command also creates a comint buffer to allow
+you to interacti with the program."
   (interactive)
-  (let ((app-class jde-run-application-class))
+  (if (string= jde-run-executable "")
+      (jde-run-main-class)
+    (jde-run-executable)))
+
+(defun jde-run-get-main-class () 
+  "Gets the main class for the application to which the current
+source buffer belongs."
+  (let ((main-class jde-run-application-class))
     (if (or
-	 (not app-class)
-	 (string= app-class ""))
-	(setq app-class
+	 (not main-class)
+	 (string= main-class ""))
+	(setq main-class
 	      (concat (jde-db-get-package)
 		      (file-name-sans-extension 
 		       (file-name-nondirectory (buffer-file-name))))))
-    (jde-run-internal app-class)))
+    main-class))
+
+
+
+(defun jde-run-main-class()
+  "Runs the Java program named by `jde-run-application-class' in
+a buffer, piping output from the program to the buffer and 
+input from the buffer to the program."
+  (interactive)
+    (jde-run-internal (jde-run-get-main-class)))
 
 (defmacro save-w32-show-window (&rest body)
-"Saves the value of the w32-start-process-show-window variable
+  "Saves the value of the w32-start-process-show-window variable
 before evaluating body and restores the value afterwards."
-  `(if (and 
-	(eq system-type 'windows-nt)
-	(boundp 'win32-start-process-show-window))
-       (let ((save win32-start-process-show-window))
-	 (setq win32-start-process-show-window t)
-	 ,@body
-	 (setq win32-start-process-show-window save))
-    ,@body))
+  `(if (and (eq system-type 'windows-nt)
+	    (not jde-xemacsp))
+       (if (boundp 'win32-start-process-show-window)
+	   (let ((save win32-start-process-show-window))
+	     (setq win32-start-process-show-window t)
+	     ,@body
+	     (setq win32-start-process-show-window save))
+	 (let ((save w32-start-process-show-window))
+	   (setq w32-start-process-show-window t)
+	   ,@body
+	   (setq w32-start-process-show-window save)))	 
+     ,@body))
 
 (defun jde-run-unquote (string)
   (if (eq (aref string 0) ?\")
       (substring string 1 (- (length string) 1))
     string))
 
+(defun jde-run-application-running-p ()
+  "*Returns t if the application to which the current
+buffer belongs is running."
+  (let ((run-buf-name (concat "*" (jde-run-get-main-class) "*")))
+    (comint-check-proc run-buf-name)))
+
 (defun jde-run-internal(app-class)
-  (let ((run-buf-name (concat "*" app-class "*")))
+  (let ((run-buf-name (concat "*" app-class "*"))
+	(source-directory default-directory)
+	(working-directory (if (string= jde-run-working-directory "")
+			       default-directory
+			     jde-run-working-directory)))
     (if (not (comint-check-proc run-buf-name))
 	(let* ((run-buffer (get-buffer-create run-buf-name))
 	       (win32-p (eq system-type 'windows-nt))
@@ -532,18 +613,18 @@ before evaluating body and restores the value afterwards."
 			       (jde-run-parse-args
 				(read-from-minibuffer
 				 "Vm args: "
-				 jde-run-interactive-vm-args
+				 (car jde-run-interactive-vm-arg-history)
 				 nil nil
-				 '(jde-run-interactive-vm-arg-history . 1))))
+				 'jde-run-interactive-vm-arg-history)))
 			   (list app-class)
 			   jde-run-option-application-args
 			   (if jde-run-read-app-args
 			       (jde-run-parse-args
 				(read-from-minibuffer
 				 "Application args: "
-				 jde-run-interactive-app-args
+				 (car jde-run-interactive-app-arg-history)
 				 nil nil
-				 '(jde-run-interactive-app-arg-history . 1))))
+				 'jde-run-interactive-app-arg-history)))
 			   ))
 	       (command-string (concat prog " " 
 				       (jde-run-make-arg-string
@@ -552,15 +633,56 @@ before evaluating body and restores the value afterwards."
 	  (save-excursion
 	    (set-buffer run-buffer)
 	    (erase-buffer)
-	    (insert (concat "cd " default-directory "\n"))
+	    (cd working-directory)
+	    (insert (concat "cd " working-directory "\n"))
 	    (insert command-string)
 	    (jde-run-mode))
 	  (save-w32-show-window
 	   (comint-exec run-buffer app-class prog nil prog-args))
-	  (pop-to-buffer run-buffer))
+	  (pop-to-buffer run-buffer)
+	  (cd source-directory))
       (message "An instance of %s is running." app-class)
       (pop-to-buffer run-buf-name))))
 
+(defun jde-run-executable()
+  (let* ((prog-name (file-name-sans-extension
+		     (file-name-nondirectory jde-run-executable)))
+	 (run-buf-name (concat "*" prog-name "*"))
+	 (source-directory default-directory)
+	 (working-directory (if (string= jde-run-working-directory "")
+				default-directory
+			      jde-run-working-directory)))
+    (if (not (comint-check-proc run-buf-name))
+	(let* ((run-buffer (get-buffer-create run-buf-name))
+	       (prog-args (append
+			   jde-run-executable-args
+			   (if jde-run-read-app-args
+			       (jde-run-parse-args
+				(read-from-minibuffer
+				 "Application args: "
+				 jde-run-interactive-app-args
+				 nil nil
+				 '(jde-run-interactive-app-arg-history . 1))))
+			   ))
+	       (command-string (concat jde-run-executable " " 
+				       (mapconcat (lambda (arg) arg)
+						  prog-args " ")
+				       "\n\n")))
+	  (save-excursion
+	    (set-buffer run-buffer)
+	    (erase-buffer)
+	    (cd working-directory)
+	    (insert (concat "cd " working-directory "\n"))
+	    (insert command-string)
+	    (jde-run-mode))
+	  (save-w32-show-window
+	   (comint-exec run-buffer prog-name jde-run-executable nil prog-args))
+	  (pop-to-buffer run-buffer)
+	  (cd source-directory))
+      (message "An instance of %s is running." prog-name)
+      (pop-to-buffer run-buf-name))))
+
+
 ;;;###autoload
 (defun jde-run-mode()
   "Mode for running Java programs."
@@ -570,7 +692,25 @@ before evaluating body and restores the value afterwards."
   (run-hooks 'jde-run-mode-hook)
 )
 
-    
+(defun jde-get-appletviewer-options ()
+  (let (options)
+    (if (not (string= jde-appletviewer-option-encoding ""))
+	(setq options (list 
+			"-encoding"
+		        jde-appletviewer-option-encoding)))
+    (if jde-appletviewer-option-vm-args
+	(let ((len (length jde-appletviewer-option-vm-args))
+	      (n 0))
+	  (while (< n len)
+	    (setq options
+		  (nconc 
+		   options
+		   (list 
+		    (concat "-J"
+			    (nth n jde-appletviewer-option-vm-args)))))
+	    (setq n (1+ n)))))
+    options))
+ 
 (defun jde-run-applet-exec (buffer name command startfile switches)
   "A version of comint-exec patched to start an applet viewer as
 a command shell subprocess rather than as a subprocess of Emacs. This
@@ -624,45 +764,176 @@ under Windows."
     (if (not (comint-check-proc run-buf-name))
 	(let* ((run-buffer (get-buffer-create run-buf-name))
 	       (win32-p (eq system-type 'windows-nt))
-	       (prog jde-run-applet-viewer)) 
+	       (prog jde-run-applet-viewer)
+	       (prog-args
+		(append (jde-get-appletviewer-options)
+			(list doc)))
+	       (command-string (concat prog " "
+				       (jde-run-make-arg-string
+					prog-args)
+				       "\n\n")))
 	  (save-excursion
 	    (set-buffer run-buffer)
 	    (erase-buffer)
 	    (insert (concat "cd " default-directory "\n"))
-	    (insert (concat prog " "  doc "\n\n"))
+	    (insert command-string)
 	    (jde-run-mode))
-	  (jde-run-applet-exec run-buffer doc-name prog nil (list doc))
+	  (jde-run-applet-exec run-buffer doc-name prog nil prog-args)
 	  (pop-to-buffer run-buffer))
-      (message "An instance of %s is running." app-class)
+      (message "An instance of the applet in %s is running." doc-name)
       (pop-to-buffer run-buf-name))))
 
+
+(defun jde-run-find-html-files ()
+  "If (buffer-file-name) is /a/b/c.xxx (where xxx can be anything), 
+return (\"/a/b/c.html\") if it exists, else return (\"/a/b/c.htm\") 
+if it exists, else return a list of all *.html files in /a/b/
+directory."
+  (let ((basename (file-name-sans-extension (buffer-file-name)))
+	f)
+    (cond 
+     ((file-exists-p (setq f (concat basename ".html")))
+      (list f))
+     ((file-exists-p (setq f (concat basename ".htm"))) ;; for poor winXX souls
+      (list f))
+     (t
+      (mapcan (lambda (file)
+		(if (or
+		     (string-match "[.]html$" file)
+		     (string-match "[.]htm$" file))
+		    (list file)))       
+	      (directory-files 
+	       (file-name-directory (buffer-file-name)) t))))))
+
+
+
+(setq jde-run-applet-document nil) 
+
 ;;;###autoload
 (defun jde-run-applet (&optional doc)
-  "Run an applet, using the viewer specified by jde-run-applet-viewer
-and the applet document specified by jde-run-applet-doc."
+  "Runs an applet. This function prompts you to enter the path of an
+html document that displays the applet. If you enter return without
+specifying a document, this function next checks whether
+`jde-run-applet-doc' specifies a document. If so, it displays that
+specified document. Next, it checks whether the current directory
+contains any html files. If the current directory contains an html
+file with the same root name as the Java file in the current buffer,
+it displays the file. If not, it displays the first html file that it
+finds in the current directory. If if cannot find an html file, it
+signals an error.  This function uses the viewer specified by
+`jde-run-applet-viewer' to display the specified document. Note that
+if you run two html applet files successively with the same name, you
+must kill the buffer created to run the first file before running the
+second file. Otherwise, this command will simply redisplay the first
+file."
   (interactive
-   "sEnter applet document name: ")
-  (let ((applet-doc (if doc
-			doc
-		      (if jde-run-applet-doc
-			  jde-run-applet-doc
-			(concat 
-			 (file-name-sans-extension buffer-file-name)
-			 ".html")))))
-	(jde-run-applet-internal 
-	 (if (string-match "appletviewer" jde-run-applet-viewer)
-	     applet-doc
-	   (concat default-directory applet-doc)))))
+   (let ((insert-default-directory nil))
+     (list (read-file-name "Applet doc: " nil nil nil jde-run-applet-document))))
+  (setq jde-run-applet-document doc)
+  (let ((applet-doc (if (and jde-run-applet-document
+			     (not (string= jde-run-applet-document "")))
+			jde-run-applet-document
+		      (if (and jde-run-applet-doc
+			       (not (string= jde-run-applet-doc "")))
+			    jde-run-applet-doc
+			  (car (jde-run-find-html-files))))))
+    (if applet-doc
+	(if (string-match "appletviewer" jde-run-applet-viewer)
+	    (jde-run-applet-internal applet-doc)
+	  (if (or
+	       (string= jde-run-applet-viewer "")
+	       (string-match "browse-url" jde-run-applet-viewer))
+	      (browse-url applet-doc browse-url-new-window-p)
+	    (jde-run-applet-internal (concat default-directory applet-doc))))
+      (signal 'error "Could not find html document to display applet."))))
+
 
 (defun jde-run-menu-run-applet ()
   (interactive)
-  (jde-run-applet jde-run-applet-doc))
+  (jde-run-applet))
+   
 
 (provide 'jde-run)
 
 
 ;; Change History
-;; $Log: jde-run.el $
+;; $Log: jde-run.el,v $
+;; Revision 1.1  2000/08/13 13:44:01  michaels
+;; Initial checkin.
+;;
+;; Revision 1.36  2000/03/16 05:18:11  paulk
+;; Miscellaneous small bug fixes and enhancements.
+;;
+;; Revision 1.35  2000/02/16 04:40:33  paulk
+;; Implemented Cygwin/XEmacs compatiblity fixes provided by Fred Hart
+;; <cfhart@Z-TEL.com> in bsh-internal.
+;;
+;; Revision 1.34  2000/02/01 04:11:56  paulk
+;; ReleaseNotes.txt
+;;
+;; Revision 1.33  1999/12/27 08:02:13  paulk
+;; Enhanced JDE->Run App command to run executables.
+;;
+;; Revision 1.32  1999/12/14 05:15:09  paulk
+;; JDE->Run Applet now looks in the current Java source directory for an
+;; html file having the same root name as the current Java source
+;; buffer. If it finds such a file, it runs it. Otherwise, it runs the first html file that it encounters in the directory. Thanks to  Richard Y. Kim <ryk@coho.net> for providing a patch implementing this change.
+;;
+;; Revision 1.31  1999/12/03 08:22:00  paulk
+;; Updated JDEbug to run under JDK 1.3beta.
+;;
+;; Revision 1.30  1999/09/28 04:06:59  paulk
+;; Supplied missing left parentheses.
+;;
+;; Revision 1.29  1999/09/05 04:33:28  paulk
+;; Added support for running vm in classic mode.
+;;
+;; Revision 1.28  1999/08/29 04:29:18  paulk
+;; Patches provided by Michael Ernst <mernst@alum.mit.edu>
+;;
+;; Revision 1.27  1999/07/04 03:31:11  paulk
+;; Added jde-run-application-running-p predicate function.
+;;
+;; Revision 1.26  1999/05/07 23:22:09  paulk
+;; Changed jde-run-parse-args to accept any substring enclosed in single or double
+;; quotes or that does not contain white space as an arg.
+;;
+;; Revision 1.25  1999/02/10 18:29:03  paulk
+;; Added support for appletviewer options.
+;;
+;; Revision 1.24  1999/02/05 21:59:30  paulk
+;; Added file-completion and default-to-last-entry to jde-run-applet
+;; command.
+;;
+;; Revision 1.23  1999/02/03 01:08:34  paulk
+;; Enhanced jde-run-applet to look in current directory for html file
+;; to display, if you do not specify a file in the minibuffer or via
+;; jde-run-applet-doc. Also fixed a bug in the minibuffer version of
+;; jde-run-applet that forced you always to specify the name of an
+;; html document.
+;;
+;; Revision 1.22  1999/01/15 21:59:34  paulk
+;; Added Andy Piper's NT/XEmacs compatibility changes.
+;;
+;; Revision 1.21  1998/12/06 02:19:36  paulk
+;; Fixed bug with jde-run-options-properties. (The bug was putting a space before the
+;; -D switch.)
+;;
+;; Revision 1.20  1998/09/11 23:53:32  paulk
+;; Added a jde-run-working-directory customization variable. If set to a valid
+;; path, the JDE starts the application from the directory specified by the
+;; path. If the value of this variable is the empty string (the default),
+;; the JDE starts the application from the default directory of the current
+;; source buffer. The default directory is usually the directory containing
+;; the source file.
+;;
+;; Revision 1.19  1998/08/28 12:49:23  paulk
+;; Updated to support NT/Emacs 20.3
+;;
+;; Revision 1.18  1998/07/02 05:36:00  paulk
+;; Added $ to the set of characters recognized by the JDE as valid
+;; in vm and Java app command-line arguments.
+;;
 ;; Revision 1.17  1998/06/30 21:10:28  paulk
 ;; Fixed jde-run-parse-args to recognize % as an argument
 ;; character.
